(define-module (alterator backend squid)
  :use-module (srfi srfi-1)
  :use-module (srfi srfi-2)
  :use-module (srfi srfi-13)
  :use-module (alterator algo)
  :use-module (alterator str)
  :use-module (alterator woo)
  :use-module (alterator plist)
  :use-module (alterator gettext)
  :use-module (alterator metalterator)
  :use-module (alterator metalterator match))

;; (define (semicolonize names options)
;;   (plist-fold (lambda (name val options)
;;                 (append options
;;                         (list name)
;;                         (if (member name names)
;;                             (list
;;                              (string-join (map sure-string
;;                                                (cond
;;                                                 ((not val) '())
;;                                                 ((list? val) val)
;;                                                 (else (list val))))
;;                                           ";"))
;;                             (list val))))
;;               '()
;;               options))

(define (backend objects options)
  (print-debug 1 "[squid] Command: ~s ~s" objects options)
  (let* ((language (string-cut (sure-value options 'language "en_US") #\;))
         (_ (make-translator "alterator-squid" language)))
    (woo-case (objects options)
      ; General settings
      ((type ())
       (list 'http_port "tcp-port"))
      ((read ())
       (meta read 'squid '()
             (list 'http_port #t 'auth-mode #t 'mode #t)))
      ((write () ('http_port @p 'auth-mode @a 'mode @m)
       (meta write 'squid '()
             (list 'http_port @p 'auth-mode @a 'mode @m))))
      ((list ('modes))
       `(("transparent" label ,(_ "Transparent"))
         ("normal" label ,(_ "Normal"))))
      ((list ('auth-modes))
       `(("no-auth" label ,(_ "No authentication"))
         ("kerb" label ,(_ "Kerberos"))))
      ((list ('proxy-methods))
       `(("PROXY" label ,(_ "Proxy"))
         ("CONNECT" label ,(_ "Passthrough"))))
      ; Port-related ACLs
      ((read ('safe-ports)) '())
      ((list ('safe-ports))
       (meta list 'squid (list 'safe-ports)
             (list 'start_port #t 'end_port #t 'method #t 'comment "label")))
      ((read-next ('safe-ports))
       (meta read-next 'squid (list 'safe-ports) (list 'next-id "id")))
      ((new ('safe-ports @n)
            ('start_port @s 'end_port @e 'method @m 'comment @c))
       (meta new 'squid (list 'safe-ports @n)
             (list 'start_port @s 'end_port @e 'method @m 'comment @c)))
      ((delete ('safe-ports @n))
       (meta delete 'squid (list 'safe-ports @n) '()))
      ((type ('safe-ports @n))
       (list 'start_port "tcp-port" 'end_port "tcp-port"))
      ((read ('safe-ports @n))
       (meta read 'squid (list 'safe-ports @n)
             (list 'start_port #t 'end_port #t 'method #t 'comment #t)))
      ((write ('safe-ports @n)
              ('start_port @s 'end_port @e 'method @m 'comment @c))
       (meta write 'squid (list 'safe-ports @n)
             (list 'start_port @s 'end_port @e 'method @m 'comment @c)))
      ; Domain-related ACLs
      ((read ('domains)) '())
      ((list ('domains))
       (meta list 'squid (list 'domains)
             (list 'suffix #t 'comment "label")))
      ((read-next ('domains))
       (meta read-next 'squid (list 'domains) (list 'next-id "id")))
      ((new ('domains @n) ('suffix @s 'comment @c))
       (meta new 'squid (list 'domains @n) (list 'suffix @s 'comment @c)))
      ((delete ('domains @n))
       (meta delete 'squid (list 'domains @n) '()))
      ((type ('domains @n))
       (list 'suffix "domain-suffix-list"))
      ((read ('domains @n))
       (meta read 'squid (list 'domains @n)
             (list 'suffix #t 'comment #t)))
      ((write ('domains @n) ('suffix @s 'comment @c))
       (meta write 'squid (list 'domains @n)
             (list 'suffix @s 'comment @c)))
      ; Network-related ACLs
      ((read ('networks)) '())
      ((list ('networks))
       (meta list 'squid (list 'networks)
             (list 'address #t 'mask #t 'comment "label")))
      ((read-next ('networks))
       (meta read-next 'squid (list 'networks) (list 'next-id "id")))
      ((new ('networks @n) ('address @a 'mask @m 'comment @c))
       (meta new 'squid (list 'networks @n)
             (list 'address @a 'mask @m 'comment @c)))
      ((delete ('networks @n))
       (meta delete 'squid (list 'networks @n) '()))
      ((type ('networks @n))
       (list 'address "ipv4-address" 'mask "ipv4-address"))
      ((read ('networks @n))
       (meta read 'squid (list 'networks @n)
             (list 'address #t 'mask #t 'comment #t)))
      ((write ('networks @n) ('address @a 'mask @m 'comment @c))
       (meta write 'squid (list 'networks @n)
             (list 'address @a 'mask @m 'comment @c)))
      ; Group-based access
      ((read ('groups)) '())
      ((list ('groups))
       (meta list 'squid (list 'groups)
             (list 'deny #t 'comment "label")))
      ((read-next ('groups))
       (meta read-next 'squid (list 'groups) (list 'next-id "id")))
      ((new ('groups @n) ('comment @c 'deny @d))
       (meta new 'squid (list 'groups @n) (list 'comment @c 'deny @d))
       (meta new 'squid (list 'groups @n 'domains) '()))
      ((delete ('groups @n))
       (meta delete 'squid (list 'groups @n) '()))
      ((type ('groups @n))
       (list 'deny "boolean"))
      ((read ('groups @n))
       (meta read 'squid (list 'groups @n) (list 'deny #t)))
      ((write ('groups @n) ('deny @d))
       (meta write 'squid (list 'groups @n) (list 'deny @d)))
      ; Group-domain relation
      ((list ('groups @g 'domains))
       (meta list 'squid (list 'groups @g 'domains)
             (list 'comment "label")))
      ((new ('groups @g 'domains @d))
       (meta link 'squid (list 'groups @g 'domains @d)
             (list 'squid 'domains @d)))
      ((delete ('groups @g 'domains @d))
       (meta delete 'squid (list 'groups @g 'domains @d) '()))
      ((read ('groups @g 'domains @d))
       (meta read 'squid (list 'groups @g 'domains @d)
             (list 'suffix #t 'comment #t)))
      (else #f))))
