(define-module (ui squid domains ajax)
  :use-module (srfi srfi-1)
  :use-module (alterator ajax)
  :use-module (alterator woo)
  :export (init))

(define last-name #f)

(define *new-obj* "*")

(define (is-new-obj? name)
  (equal? name *new-obj*))

(define (refresh-domain name)
  (let* ((domain-list (append
                       (woo-list "/squid/domains")
                       `((,*new-obj* name ,*new-obj* label "*"))))
         (name (if (and name
                        (or (is-new-obj? name)
                            (find (lambda (domain)
                                    (equal? (woo-get-option domain 'name)
                                            name))
                                  domain-list)))
                   name
                   (woo-get-option (car domain-list) 'name)))
         (domain-data (if (is-new-obj? name)
                        '()
                        (woo-read-first
                         (string-append "/squid/domains/" name)))))
    (form-update-enum "name" domain-list)
    (form-update-value "name" name)
    (form-update-value "suffix"
                       (woo-get-option domain-data 'suffix))
    (form-update-value "comment"
                       (woo-get-option domain-data 'comment))
    (form-update-activity "delete" (not (is-new-obj? name)))
    (set! last-name name)))

(define (on-domain-change)
  (refresh-domain (or (form-value "name") last-name)))

(define (on-domain-save)
  (let ((name (form-value "name"))
        (suffix (form-value "suffix"))
        (comment (form-value "comment")))
    (cond
     ((is-new-obj? name)
      (let* ((answer (woo "read-next" "/squid/domains"))
             (name (string-append "domain-"
                                  (number->string
                                   (woo-get-option (car answer) 'id)))))
        (woo-new (string-append "/squid/domains/" name)
                 'suffix suffix
                 'comment comment)
        (refresh-domain name)))
     (else
      (catch/message (lambda ()
                       (woo-write (string-append "/squid/domains/" name)
                                  'suffix suffix
                                  'comment comment)))
      (refresh-domain name)))))

(define (go-to-new)
  (refresh-domain *new-obj*))

(define (delete-domain)
  (let ((name (form-value "name")))
    (cond
     ((not (is-new-obj? name))
      (woo-delete (string-append "/squid/domains/" name))
      (on-domain-change)))))

(define (init)
  (form-bind "name"  "change" on-domain-change)
  (form-bind "apply" "click" on-domain-save)
  (form-bind "new" "click" go-to-new)
  (form-bind "delete" "click" delete-domain)
  (form-bind "back" "click" (lambda () (form-replace "/squid/groups")))
  (on-domain-change))
