(define-module (ui squid groups ajax)
  :use-module (srfi srfi-1)
  :use-module (alterator ajax)
  :use-module (alterator woo)
  :export (init))

(define last-name #f)

(define (delete-by-name from what)
  (filter (lambda (obj1)
            (not (find (lambda (obj2)
                         (equal? (woo-get-option obj1 'name)
                                 (woo-get-option obj2 'name)))
                       what)))
          from))

(define (init-group name)
  (let ((activity (and name
                       (not (string-null? name)))))
    (form-update-activity "apply" activity)
    (form-update-activity "delete_group" activity)
    (form-update-activity "domain" #f)
    (form-update-enum "domain" '())
    (form-update-activity "selected" #f)
    (form-update-activity "add_domain" #f)
    (form-update-activity "delete_domain" #f)
    (form-update-activity "deny" activity)))

(define (refresh-group name)
  (let* ((group-list (woo-list "/squid/groups"))
         (name (and group-list
                    (not (null? group-list))
                    (if (and name
                             (find (lambda (group)
                                     (equal? (woo-get-option group 'name)
                                             name))
                                   group-list))
                        name
                        (woo-get-option (car group-list) 'name))))
         (system-group-list (delete-by-name
                             (woo-list "/ldap-groups/avail_groups")
                             group-list))
         (group (or (form-value "group")
                    (woo-get-option (car system-group-list) 'name))))
    (form-update-enum "group" system-group-list)
    (form-update-value "group" group)
    (form-update-enum "name" group-list)
    (init-group name)
    (if name
        (let* ((group-data (woo-read-first
                             (string-append "/squid/groups/" name)))
               (selected-list (woo-list
                               (string-append "/squid/groups/"
                                              name "/domains")))
               (selected (and selected-list
                               (not (null? selected-list))
                               (or (form-value "selected")
                                   (woo-get-option
                                    (car selected-list) 'name))))
               (domain-list (delete-by-name
                             (woo-list "/squid/domains")
                             selected-list))
               (domain (and domain-list
                            (not (null? domain-list))
                            (or (form-value "domain")
                                (woo-get-option
                                 (car domain-list) 'name)))))
          (form-update-value "name" name)
          (form-update-enum "selected" selected-list)
          (and (not (null? selected-list))
               (form-update-activity "selected" #t))
          (and selected
               (form-update-value "selected" selected)
               (form-update-activity "delete_domain" #t))
          (form-update-enum "domain" domain-list)
          (and (not (null? domain-list))
               (form-update-activity "domain" #t)
               (form-update-activity "add_domain" #t))
          (if domain
              (form-update-value "domain" domain))
          (form-update-value "deny" (woo-get-option group-data 'deny))
          (set! last-name name)))))

(define (on-group-change)
  (refresh-group (or (form-value "name") last-name)))

(define (on-group-save)
  (let ((name (form-value "name"))
        (deny (form-value "deny")))
    (catch/message (lambda ()
                     (woo-write (string-append "/squid/groups/" name)
                                'deny deny)))
    (refresh-group name)))

(define (add-group)
  (let ((name (form-value "group")))
    (woo-new (string-append "/squid/groups/" name) 'comment name 'deny #f)
    (refresh-group name)))

(define (delete-group)
  (let ((name (form-value "name")))
    (woo-delete (string-append "/squid/groups/" name))
    (on-group-change)))

(define (add-domain)
  (let* ((name (form-value "name"))
         (domain (form-value "domain")))
    (woo-new (string-append "/squid/groups/" name
                            "/domains/" domain))
    (refresh-group name)))

(define (delete-domain)
  (let* ((name (form-value "name"))
         (domain (form-value "selected")))
    (woo-delete (string-append "/squid/groups/" name
                               "/domains/" domain))
    (refresh-group name)))

(define (init)
  (form-bind "name"  "change" on-group-change)
  (form-bind "apply" "click" on-group-save)
  (form-bind "add_group" "click" add-group)
  (form-bind "delete_group" "click" delete-group)
  (form-bind "add_domain" "click" add-domain)
  (form-bind "domains" "click"
    (lambda ()
      (form-replace "/squid/domains")))
  (form-bind "delete_domain" "click" delete-domain)
  (form-bind "back" "click" (lambda () (form-replace "/squid")))
  (on-group-change))
