(define-module (ui squid networks ajax)
  :use-module (srfi srfi-1)
  :use-module (alterator ajax)
  :use-module (alterator woo)
  :export (init))

(define last-name #f)

(define *new-obj* "*")

(define (is-new-obj? name)
  (equal? name *new-obj*))

(define (refresh-network name)
  (let* ((network-list (append
                        (woo-list "/squid/networks")
                        `((,*new-obj* name ,*new-obj* label "*"))))
         (name (if (and name
                        (or (is-new-obj? name)
                            (find (lambda (network)
                                    (equal? (woo-get-option network 'name)
                                            name))
                                  network-list)))
                   name
                   (woo-get-option (car network-list) 'name)))
         (network-data (if (is-new-obj? name)
                        '()
                        (woo-read-first
                         (string-append "/squid/networks/" name)))))
    (form-update-enum "name" network-list)
    (form-update-value "name" name)
    (form-update-value "address"
                       (woo-get-option network-data 'address))
    (form-update-value "mask"
                       (woo-get-option network-data 'mask))
    (form-update-value "comment"
                       (woo-get-option network-data 'comment))
    (form-update-activity "delete" (not (is-new-obj? name)))
    (set! last-name name)))

(define (on-network-change)
  (refresh-network (or (form-value "name") last-name)))

(define (on-network-save)
  (let ((name (form-value "name"))
        (address (form-value "address"))
        (mask (form-value "mask"))
        (comment (form-value "comment")))
    (cond
     ((is-new-obj? name)
      (let* ((answer (woo "read-next" "/squid/networks"))
             (name (string-append "network-"
                                  (number->string
                                   (woo-get-option (car answer) 'id)))))
        (woo-new (string-append "/squid/networks/" name)
                 'address address
                 'mask mask
                 'comment comment)
        (refresh-network name)))
     (else
      (catch/message (lambda ()
                       (woo-write (string-append "/squid/networks/" name)
                                  'address address
                                  'mask mask
                                  'comment comment)))
      (refresh-network name)))))

(define (go-to-new)
  (refresh-network *new-obj*))

(define (delete-network)
  (let ((name (form-value "name")))
    (cond
     ((not (is-new-obj? name))
      (woo-delete (string-append "/squid/networks/" name))
      (on-network-change)))))

(define (init)
  (form-bind "name"  "change" on-network-change)
  (form-bind "apply" "click" on-network-save)
  (form-bind "new" "click" go-to-new)
  (form-bind "delete" "click" delete-network)
  (form-bind "back" "click" (lambda () (form-replace "/squid")))
  (on-network-change))
