(define-module (ui squid safe-ports ajax)
  :use-module (srfi srfi-1)
  :use-module (alterator ajax)
  :use-module (alterator woo)
  :export (init))

(define last-name #f)

(define *new-obj* "*")

(define (is-new-obj? name)
  (equal? name *new-obj*))

(define (refresh-port name)
  (let* ((port-list (append
                     (woo-list "/squid/safe-ports")
                     `((,*new-obj* name ,*new-obj* label "*"))))
         (name (if (and name
                        (or (is-new-obj? name)
                            (find (lambda (port)
                                    (equal? (woo-get-option port 'name)
                                            name))
                                  port-list)))
                   name
                   (woo-get-option (car port-list) 'name)))
         (port-data (if (is-new-obj? name)
                        '()
                        (woo-read-first
                         (string-append "/squid/safe-ports/" name)))))
    (form-update-enum "name" port-list)
    (form-update-value "name" name)
    (form-update-value "start_port"
                       (woo-get-option port-data 'start_port))
    (form-update-value "end_port"
                       (woo-get-option port-data 'end_port))
    (form-update-enum "method" (woo-list "/squid/proxy-methods"))
    (form-update-value "method"
                       (woo-get-option port-data 'method))
    (form-update-value "comment"
                       (woo-get-option port-data 'comment))
    (form-update-activity "delete" (not (is-new-obj? name)))
    (set! last-name name)))

(define (on-port-change)
  (refresh-port (or (form-value "name") last-name)))

(define (on-port-save)
  (let ((name (form-value "name"))
        (start_port (form-value "start_port"))
        (end_port (form-value "end_port"))
        (method (form-value "method"))
        (comment (form-value "comment")))
    (cond
     ((is-new-obj? name)
      (let* ((answer (woo "read-next" "/squid/safe-ports"))
             (name (string-append "port-"
                                  (number->string
                                   (woo-get-option (car answer)
                                                   'id)))))
        (woo-new (string-append "/squid/safe-ports/" name)
                 'start_port start_port
                 'end_port end_port
                 'method method
                 'comment comment)
        (refresh-port name)))
     (else
      (catch/message (lambda ()
                       (woo-write (string-append "/squid/safe-ports/" name)
                                  'start_port start_port
                                  'end_port end_port
                                  'method method
                                  'comment comment)))
      (refresh-port name)))))

(define (go-to-new)
  (refresh-port *new-obj*))

(define (delete-port)
  (let ((name (form-value "name")))
    (cond
     ((not (is-new-obj? name))
      (woo-delete (string-append "/squid/safe-ports/" name))
      (on-port-change)))))

(define (init)
  (form-bind "name"  "change" on-port-change)
  (form-bind "apply" "click" on-port-save)
  (form-bind "new" "click" go-to-new)
  (form-bind "delete" "click" delete-port)
  (form-bind "back" "click" (lambda () (form-replace "/squid")))
  (on-port-change))
